"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var types_1 = require("@babel/types");
var configuration_1 = tslib_1.__importDefault(require("./configuration"));
var replace_1 = require("./replace");
function isStyledTagged(tagged) {
    var tag = tagged.tag;
    if (types_1.isIdentifier(tag)) {
        return isStyled(tag);
    }
    else if (types_1.isMemberExpression(tag) && types_1.isIdentifier(tag.object)) {
        return isStyled(tag.object);
    }
    else if (types_1.isCallExpression(tag)) {
        if (types_1.isIdentifier(tag.callee)) {
            return isStyled(tag.callee);
        }
        else if (types_1.isMemberExpression(tag.callee)) {
            return isStyledMember(tag.callee);
        }
    }
    return false;
}
function isStyledMember(member) {
    if (types_1.isIdentifier(member.object)) {
        return isStyled(member.object);
    }
    else if (types_1.isMemberExpression(member.object)) {
        return isStyledMember(member.object);
    }
    return false;
}
function isStyled(id) {
    return configuration_1.default.config.tags.indexOf(id.name) >= 0;
}
function transform(template) {
    template.quasis.forEach(function (it) {
        if (it.value && it.value.raw) {
            it.value.raw = replace_1.replace(it.value.raw);
        }
        if (it.value && it.value.cooked) {
            it.value.cooked = replace_1.replace(it.value.cooked);
        }
    });
}
exports.default = helper_plugin_utils_1.declare(function (api, options) {
    api.assertVersion(7);
    console.log('custom options:', options);
    configuration_1.default.updateConfig(options);
    var visitor = {
        TaggedTemplateExpression: function (path) {
            if (isStyledTagged(path.node)) {
                transform(path.node.quasi);
            }
        },
        CallExpression: function (path) {
            if (types_1.isMemberExpression(path.node.callee) &&
                types_1.isIdentifier(path.node.callee.object) &&
                isStyled(path.node.callee.object)) {
                if (path.node.arguments.length >= 1 && types_1.isArrowFunctionExpression(path.node.arguments[0])) {
                    var arrowFn = path.node.arguments[0];
                    if (types_1.isTemplateLiteral(arrowFn.body)) {
                        transform(arrowFn.body);
                    }
                }
            }
        },
    };
    return {
        name: 'styled-components-px2vw',
        visitor: visitor,
    };
});
